dnl -*- m4 -*-
dnl----------------------------------------------------------------------
dnl This is a common set of macros to be used in the configuration of
dnl	the LDAS software
dnl----------------------------------------------------------------------

dnl======================================================================
dnl Get the list of recursive targets automatically supplied by automake
dnl======================================================================

AC_DEFUN([LDAS_CHECK_AUTOMAKE_RECURSIVE_TARGETS],
  [AC_CACHE_CHECK([find recursive targets used by automake],
    [ldas_cv_automake_recursive_targets],
    [ dnl ---------------------------------------------------------------
      dnl  Find all of the recursive targets
      dnl ---------------------------------------------------------------
     ldas_cv_automake_recursive_targets="`egrep -e '^(.*):.*\1-recursive$' ${srcdir}/Makefile.in | egrep -v -e '^(mostlyclean|clean|distclean|maintainer-clean):' | sed -e 's/:.*$/-recursive/' | tr '\n' ' '`"
    ])
  PROJECT_RECURSIVE_TARGETS="${ldas_cv_automake_recursive_targets} $PROJECT_RECURSIVE_TARGETS"
  AC_SUBST(PROJECT_RECURSIVE_TARGETS)]
)

dnl======================================================================
dnl LDAS_CHECK_PTHREAD_READ_WRITE_LOCK - See if anything special has
dnl	to be done to compile pthread's read/write locks
dnl======================================================================

AC_DEFUN([LDAS_CHECK_PTHREAD_READ_WRITE_LOCK],
[
AC_CHECK_HEADER(pthread.h)
AC_MSG_CHECKING(for additional flags to compile read/write locking for threads)
AC_CACHE_VAL(ldas_cv_pthread_read_write_lock_option,
[ldas_cv_pthread_read_write_lock_option=no
AC_LANG_SAVE
AC_LANG_C
ac_save_CFLAGS="$CFLAGS"
ac_save_CXXFLAGS="$CXXFLAGS"
for ac_arg in ""
do
  CFLAGS="$ac_save_CFLAGS $ac_arg"
  AC_TRY_COMPILE(
  [#include <pthread.h>
  ],[
  pthread_rwlock_t l;
  pthread_rwlock_init( &l, (pthread_rwlockattr_t*)NULL );
  ],
  [ldas_cv_pthread_read_write_lock_option="$ac_arg"; break; ],
  [ldas_cv_pthread_read_write_lock_option="unsupported"])
done
CFLAGS="$ac_save_CFLAGS"
])
if test -z "$ldas_cv_pthread_read_write_lock_option"
then
  AC_MSG_RESULT([none needed])
else
  AC_MSG_RESULT([$ldas_cv_pthread_read_write_lock_option])
fi
case "x$ldas_cv_pthread_read_write_lock_option" in
  xno|xunsupported) ;;
  x)
    AC_DEFINE([HAVE_PTHREAD_RW_LOCK],
	      [1],
	      [Defined if pthread read/write locks supported])
    ;;
  *)
    AC_DEFINE([HAVE_PTHREAD_RW_LOCK],
	      [1],
	      [Defined if pthread read/write locks supported])
    CFLAGS="$CFLAGS $ldas_cv_pthread_read_write_lock_option"
    CXXFLAGS="$CXXFLAGS $ldas_cv_pthread_read_write_lock_option"
    ;;
esac
AC_LANG_RESTORE
])

dnl======================================================================
dnl LDAS_CALC_VERSION - Calculate the version number
dnl======================================================================

AC_DEFUN([LDAS_INIT_AUTOMAKE],
[
if test x$LDAS_VERSION = x
then
	LDAS_VERSION=$2
	export LDAS_VERSION
fi
AM_INIT_AUTOMAKE($1,$2)
])

dnl======================================================================
dnl AC_LDAS_FINISH - Things to be done at the very end for LDAS
dnl======================================================================

AC_DEFUN([AC_LDAS_FINISH],
[
  dnl -------------------------------------------------------------------
  dnl Check for large file support
  dnl -------------------------------------------------------------------
  AC_SYS_LARGEFILE
  dnl -------------------------------------------------------------------
  dnl -- Check for documentation tools
  dnl -------------------------------------------------------------------
  LDAS_PROG_DOXYGEN
  dnl -------------------------------------------------------------------
  dnl -- Check for the presence of Valgrind
  dnl -------------------------------------------------------------------
  LDAS_PROG_VALGRIND
  LDAS_LIB_UMEM
  LDAS_LIB_STLPORT_FINISH
  AC_SUBST(LDAS_EMPTY)
  dnl ===================================================================
  dnl  Specify memory checker
  dnl ===================================================================
  AC_ARG_WITH([memory-checking],[  --with-memory-checking[[=no]]  enable extended runtime memory checking],
	      [],
              [with_memory_checking="no"])
  case x$with_memory_checking in
  xno)
    AC_MSG_CHECKING(for memory checking)
    AC_MSG_RESULT(no)
    ;;
  x|xyes)
    AC_LANG_PUSH(C++)
    AC_MSG_CHECKING(for memory checking C++)
    ldas_memory_checking_flags=""
    ldas_orig_CXXFLAGS="$CXXFLAGS"
    dnl -----------------------------------------------------------------
    dnl  -fmudflapth
    dnl -----------------------------------------------------------------
    CXXFLAGS="-fmudflapth $ldas_orig_CXXFLAGS"
    AC_RUN_IFELSE([int main( ) { return( 0 ); }],
	          [ ldas_memory_checking_flags="-fmudflapth $ldas_memory_checking_flags" ],[],[])
    dnl -----------------------------------------------------------------
    dnl  -bounds-check
    dnl -----------------------------------------------------------------
    CXXFLAGS="-fbounds-check $ldas_orig_CXXFLAGS"
    AC_RUN_IFELSE([int main( ) { return( 0 ); }],
	          [ ldas_memory_checking_flags="-fbounds-check $ldas_memory_checking_flags" ],[],[])
    dnl -----------------------------------------------------------------
    dnl  -stack-check
    dnl -----------------------------------------------------------------
    CXXFLAGS="-fstack-check $ldas_orig_CXXFLAGS"
    AC_RUN_IFELSE([int main( ) { return( 0 ); }],
	          [ ldas_memory_checking_flags="-fstack-check $ldas_memory_checking_flags" ],[],[])
    dnl -----------------------------------------------------------------
    dnl Finish with memory checks
    dnl -----------------------------------------------------------------
    CXXFLAGS="$ldas_memory_checking_flags $ldas_orig_CXXFLAGS"
    AC_MSG_RESULT($ldas_memory_checking_flags)
    AC_LANG_POP(C++)
    AC_LANG_PUSH(C)
    AC_MSG_CHECKING(for memory checking C)
    ldas_memory_checking_flags=""
    ldas_orig_CFLAGS="$CFLAGS"
    dnl -----------------------------------------------------------------
    dnl  -fmudflapth
    dnl -----------------------------------------------------------------
    CFLAGS="-fmudflapth $ldas_orig_CFLAGS"
    AC_RUN_IFELSE([int main( ) { return( 0 ); }],
	          [ ldas_memory_checking_flags="-fmudflapth $ldas_memory_checking_flags" ],[],[])
    dnl -----------------------------------------------------------------
    dnl  -bounds-check
    dnl -----------------------------------------------------------------
    CFLAGS="-fbounds-check $ldas_orig_CFLAGS"
    AC_RUN_IFELSE([int main( ) { return( 0 ); }],
	          [ ldas_memory_checking_flags="-fbounds-check $ldas_memory_checking_flags" ],[],[])
    dnl -----------------------------------------------------------------
    dnl  -stack-check
    dnl -----------------------------------------------------------------
    CFLAGS="-fstack-check $ldas_orig_CFLAGS"
    AC_RUN_IFELSE([int main( ) { return( 0 ); }],
	          [ ldas_memory_checking_flags="-fstack-check $ldas_memory_checking_flags" ],[],[])
    dnl -----------------------------------------------------------------
    dnl Finish with memory checks
    dnl -----------------------------------------------------------------
    CFLAGS="$ldas_memory_checking_flags $ldas_orig_CFLAGS"
    AC_MSG_RESULT($ldas_memory_checking_flags)
    AC_LANG_POP(C)
    ;;
  esac
  dnl ===================================================================
  dnl  Specify Arch optimiations
  dnl ===================================================================
  AC_MSG_CHECKING(for architecture optimization)
  AC_ARG_WITH([opt-arch],[  --with-opt-arch[[=no]]            enable cpu archtecture optimization],
                [],
                [with_opt_arch="no"])
  case x$with_opt_arch in
  xno)
    ;;
  x|xyes)
    case "`uname -m`" in
    i686)
      opt_arch="pentium4";
      ;;
    sun4u)
      opt_arch="v8plus";
      ;;
    esac
    ;;
  *)
    opt_arch="$with_opt_arch";
    ;;
  esac
  case x$with_opt_arch in
  xno)
    ;;
  *)
    case x$LDAS_CC_FLAVOR in
    xgnu)
      case "`uname -m`" in
      i*86)
        opt_arch_flag="-march=";
        ;;
      sun4u)
        opt_arch_flag="-mfpu -mno-unaligned-doubles -Wa,-xarch=v8plusb -m";
        ;;
      esac
      CFLAGS="$opt_arch_flag$opt_arch $CFLAGS"
      ;;
    xSunPRO)
      case x$opt_arch in
      xv8plus)
        CFLAGS="-xarch=${opt_arch}a $CFLAGS"
        ;;
      esac
      ;;
    esac
    case x$LDAS_CXX_FLAVOR in
    xgnu)
      case "`uname -m`" in
      i*86)
        opt_arch_flag="-march=";
        ;;
      sun4u)
        opt_arch_flag="-mfpu -mno-unaligned-doubles -Wa,-xarch=v8plusb -m";
        ;;
      esac
      CXXFLAGS="$opt_arch_flag$opt_arch $CXXFLAGS"
      ;;
    xSunPRO)
      case x$opt_arch in
      xv8plus)
        CXXFLAGS="-xarch=${opt_arch}a $CXXFLAGS"
        ;;
      esac
      ;;
    esac
    ;;
  esac
  case x$opt_arch in
  x)
    AC_MSG_RESULT(none)
    ;;
  *)
    AC_MSG_RESULT($opt_arch)
    ;;
  esac

  dnl ===================================================================
  dnl  Specify warning message level
  dnl ===================================================================
  CFLAGS="${ldas_prog_cc_warning} ${CFLAGS}"
  CXXFLAGS="${ldas_prog_cxx_warning} ${CXXFLAGS}"

if test "x$LDAS_LIBTOOL_VERSION_FLAG" = x
then
  LDAS_LIBTOOL_VERSION_FLAG="`echo ${LDAS_VERSION} | sed -e 's/\([[0-9]][[0-9]]*\)\.\([[0-9]][[0-9]]*\)\.\([[0-9]][[0-9]]*\)/-release \1 -version-info \2:\3:0/g'`";
  export LDAS_LIBTOOL_VERSION_FLAG;
fi
if test "x$LDAS_SO" = x
then
  LDAS_SO="`echo ${LDAS_VERSION} | sed -e 's/\([[0-9]][[0-9]]*\)\.\([[0-9]][[0-9]]*\)\.\([[0-9]][[0-9]]*\)/-\1.so.\2.0.\3/g'`";
  export LDAS_SO;
fi
dnl =====================================================================
dnl  Optimization
dnl =====================================================================
AC_LDAS_ARG_WITH_OPTIMIZATION
dnl -----------------------------------------------------------------------
CFLAGS="$LDAS_DEFS $CFLAGS"
echo "$CFLAGS" | egrep -e ${ldas_prog_cc_optimization_key}'[[0-9]]'
if test $? -ne 0
then
  CFLAGS="$CFLAGS ${ldas_prog_cc_optimization}"
else
  CFLAGS="`echo $CFLAGS | sed -e 's/'${ldas_prog_cc_optimization_key}'[[0-9]]/'${ldas_prog_cc_optimization}'/g'`"
fi
dnl -----------------------------------------------------------------------
echo "$LDAS_DEFS $CXXFLAGS" | egrep -e ${ldas_prog_cxx_optimization_key}'[[0-9]]'
if test $? -ne 0
then
  CXXFLAGS="$LDAS_DEFS $CXXFLAGS ${ldas_prog_cxx_optimization}"
else
  CXXFLAGS="`echo $LDAS_DEFS $CXXFLAGS | sed -e 's/'${ldas_prog_cxx_optimization_key}'[[0-9]]/'${ldas_prog_cxx_optimization}'/g'`"
fi
dnl -----------------------------------------------------------------------
dnl -- figure out which directories really exist
dnl -----------------------------------------------------------------------
if test -n "${subdirs}"
then
  for dir in ${subdirs}
  do
    if test -d ${srcdir}/${dir}
    then
      ldas_finish_subdirs="${ldas_finish_subdirs} ${dir}"
    fi
  done
  subdirs="${ldas_finish_subdirs}"
fi

dnl -- thread safety
AC_ARG_WITH(
        threads,
        [  --without-threads       Don't build reentrant code ],
        [ case x"$with_threads" in
          "x"|xno) ;;
          *)
		CFLAGS="${CFLAGS} -D_REENTRANT -D_POSIX_PTHREAD_SEMANTICS"
		CXXFLAGS="${CXXFLAGS} -D_REENTRANT -D_POSIX_PTHREAD_SEMANTICS"
		LDAS_CHECK_PTHREAD_READ_WRITE_LOCK
		;;
	  esac
	],[
	  CFLAGS="${CFLAGS} -D_REENTRANT -D_POSIX_PTHREAD_SEMANTICS"
	  CXXFLAGS="${CXXFLAGS} -D_REENTRANT -D_POSIX_PTHREAD_SEMANTICS"
	  LDAS_CHECK_PTHREAD_READ_WRITE_LOCK
	])


dnl -- check for unrolling of inlined functions
AC_ARG_ENABLE(inlining,
	[ --disable-inlining	Build without inlining],
	[],
	[enable_inlining=default])

case x${enable_inlining} in
xno)
  if test "${GCC}" = "yes"
  then
    CFLAGS="${CFLAGS} -fno-inline"
  fi
  if test "${GXX}" = "yes"
  then
    CXXFLAGS="${CXXFLAGS} -fno-inline"
  fi
  ;;
xyes)
  if test "${GCC}" = "yes"
  then
    FLAGS="${CFLAGS} -finline"
  fi
  if test "${GXX}" = "yes"
  then
    CXXFLAGS="${CXXFLAGS} -finline"
  fi
  ;;
*)
  ;;
esac

dnl -- rearrange prefix directories

CXXFLAGS="`echo $CXXFLAGS |  tr ' \011' '\012\012' | grep -v -- '-[[IL]]'${prefix} |  tr '\012' ' '``echo $CXXFLAGS | tr ' \011' '\012\012' | grep -- '-[[IL]]'${prefix} | sort | uniq | tr '\012' ' '`"
CPPFLAGS="`echo $CPPFLAGS |  tr ' \011' '\012\012' | grep -v -- '-[[IL]]'${prefix} |  tr '\012' ' '``echo $CPPFLAGS | tr ' \011' '\012\012' | grep -- '-[[IL]]'${prefix} | sort | uniq | tr '\012' ' '`"
LDFLAGS="`echo $LDFLAGS |  tr ' \011' '\012\012' | grep -v -- '-[[IL]]'${prefix} |  tr '\012' ' '``echo $LDFLAGS | tr ' \011' '\012\012' | grep -- '-[[IL]]'${prefix} | sort | uniq | tr '\012' ' '`"

AC_LDAS_WITH_PROFILE
AC_LDAS_WITH_LDAS_TARGET
AC_LDAS_WITH_MOUNT_PT
AC_SUBST(LDAS_PACKAGE_NAME)
AC_SUBST(LDAS_PACKAGE_VERSION)
AC_SUBST(LDAS_LIBTOOL_VERSION_FLAG)
AC_SUBST(LDAS_SO)
])

dnl======================================================================
dnl AC_LDAS_INIT([top]) - Initializes the LDAS macro set. If top is
dnl	passed to the macro, then the current directory will be 
dnl	taken as the top most directory of the LDAS system.
dnl======================================================================

AC_DEFUN([AC_LDAS_INIT],
[
  dnl -------------------------------------------------------------------
  dnl  Information about the current system for which LDAS is being
  dnl    configured.
  dnl -------------------------------------------------------------------
  ldas_kernel=`uname -s`
  ldas_processor=`uname -p`
  ldas_hardware=`uname -i`
  dnl -------------------------------------------------------------------
  dnl Misc. Info
  dnl -------------------------------------------------------------------
  ldas_top=""
  if test -n "$1" && test "$1" = "top"
  then
    ldas_top="."
    touch .ldas_top
  else
    for dir in .. ../.. ../../.. ../../../..
    do
      if test -f ${dir}/.ldas_top
      then
  	ldas_top="${dir}"
  	break
      fi
    done
  fi
  ldas_top_srcdir='[$][(]top_srcdir[)]/'${ldas_top}
  ldas_top_builddir='[$][(]top_builddir[)]/'${ldas_top}
  dnl -------------------------------------------------------------------
  dnl  Check if user has custom requests
  dnl -------------------------------------------------------------------
  AC_LDAS_WITH_EXTRA_CXXFLAGS
  AC_LDAS_WITH_EXTRA_LDFLAGS
  AC_LDAS_ENABLE_AUTO_LDCG_INCLUSION
  AC_LDAS_PROG_CC
  dnl -- Automatically add /ldcg into the build
  if test x$enable_auto_ldcg_inclusion == xyes && test -d /ldcg
  then
    case x$enable_64bit in
    xyes)
      AC_LDAS_VAR_APPEND(CFLAGS,/ldcg/include64)
      AC_LDAS_VAR_APPEND(CXXFLAGS,/ldcg/include64)
      if test -d /ldcg/lib/64 || test -h /ldcg/lib/64
      then
        AC_LDAS_VAR_APPEND(LDFLAGS,[-Wl,${RPATH},/ldcg/lib/64])
        AC_LDAS_VAR_APPEND(LDFLAGS,-L/ldcg/lib/64)
      else
        AC_LDAS_VAR_APPEND(LDFLAGS,[-Wl,${RPATH},/ldcg/lib64])
        AC_LDAS_VAR_APPEND(LDFLAGS,-L/ldcg/lib64)
      fi
      ;;
    esac
    AC_LDAS_VAR_APPEND(CFLAGS,/ldcg/include)
    AC_LDAS_VAR_APPEND(CXXFLAGS,/ldcg/include)
    AC_LDAS_VAR_APPEND(LDFLAGS,[-Wl,${RPATH},/ldcg/lib])
    AC_LDAS_VAR_APPEND(LDFLAGS,-L/ldcg/lib)
  fi
  AC_LDAS_WITH_LDFLAGS
  dnl -------------------------------------------------------------------
  dnl Export meaningful information
  dnl -------------------------------------------------------------------
  AC_SUBST(ldas_top)
  AC_SUBST(ldas_top_srcdir)
  AC_SUBST(ldas_top_builddir)

  dnl -------------------------------------------------------------------
  dnl Some variable that may or may not be set, but will be used.
  dnl -------------------------------------------------------------------
  AC_SUBST(EXTRA_LIBS)
  LDAS_LIB_STLPORT
  dnl -------------------------------------------------------------------
  dnl  Additional recursive targets
  dnl -------------------------------------------------------------------
  LDAS_CHECK_AUTOMAKE_RECURSIVE_TARGETS
  PROJECT_RECURSIVE_TARGETS="memcheck-recursive $PROJECT_RECURSIVE_TARGETS"
])

dnl======================================================================
dnl Check for the enabling of certain flags
dnl======================================================================
AC_DEFUN([AC_LDAS_ARG_ENABLE_64BIT],[
  dnl''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
  dnl Check if 
  dnl''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
  AC_ARG_ENABLE(64bit,
	[ --enable-64bit	Build 64-bit binaries],
	[enable_64bit="yes"],
	[enable_64bit="no"])
  case "${enable_64bit}" in
  yes)
    dnl''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    dnl Setup Linker
    dnl''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    if test $ldas_kernel = SunOS
    then
      if test -d /ldcg
      then 
        if test $ldas_kernel = SunOS
        then
	  AC_LDAS_VAR_PREPEND(LDFLAGS,-L/ldcg/lib/64)
          AC_LDAS_VAR_PREPEND(LDFLAGS,[-Wl,${RPATH},/ldcg/lib/64])
        fi
      fi
    fi
    dnl AC_LDAS_VAR_PREPEND(LDFLAGS,-m64)
    dnl :TODO: Verify if the following line is even needed
    dnl AC_LDAS_VAR_PREPEND(LIBS,[-Wl,-m64])
    ;;
  *)
    ;;
  esac # enable_64bit
])

dnl======================================================================
dnl Checking for the existence of certain libraries
dnl======================================================================

AC_DEFUN([AC_LDAS_CHECK_LIB_Z],
[ have_libz='yes'
  AC_CHECK_LIB( 
	z,
	compress,
	[ AC_DEFINE( HAVE_LIBZ,1,[Defined if have libz] )
	  AC_CHECK_LIB(z, compress2,[AC_DEFINE( HAVE_LIBZ_COMPRESS2,1,[Defined if libz had compress2 function] )]) ],
	[ if test -n "$1" && test "$1" = "no"
	  then
		AC_MSG_WARN( Required zlib library (libz.a) not found ) 
	        have_libz='';
	  else
		AC_MSG_ERROR( Required zlib library (libz.a) not found ) 
	  fi
	])
  AM_CONDITIONAL(HAVE_LIBZ, test x$have_libz != x)
])


AC_DEFUN([AC_LDAS_CHECK_LIB_BZ2],
[ have_libbz2='yes'
  AC_CHECK_LIB(bz2,BZ2_bzBuffToBuffCompress,
  [ AC_DEFINE( HAVE_NEW_LIBZ2,1,[Defined if libz2 has new functions] ) ],
  [ AC_CHECK_LIB(bz2, bzBuffToBuffCompress,,
    [ if test -n "$1" && test "$1" = "no"
      then
        AC_MSG_WARN( Library libbz2 not found )
	have_libbz2=''
      else
        AC_MSG_ERROR( Required library libbz2 not found )
      fi
    ])
  ])
  AM_CONDITIONAL(HAVE_LIBBZ2, test x$have_libbz2 != x)
])

   
dnl======================================================================
dnl Checking arguments that were passed to configure
dnl======================================================================

dnl----------------------------------------------------------------------
dnl Routines for appending to certain variables
dnl----------------------------------------------------------------------

AC_DEFUN([AC_LDAS_PREFIX_LDFLAGS],
[ for lib in $1
  do
    AC_LDAS_VAR_PREPEND(LDFLAGS,${lib})
  done
])

AC_DEFUN([AC_LDAS_PREFIX_INCLUDE_PATH],
[ for inc in $1
  do
    AC_LDAS_VAR_APPEND(CPPFLAGS,${inc},-I)
  done
])

dnl======================================================================
dnl AC_LDAS_ENABLE_AUTO_LDCG_INCLUSION
dnl======================================================================

AC_DEFUN([AC_LDAS_ENABLE_AUTO_LDCG_INCLUSION],
[AC_ARG_ENABLE([auto-ldcg-inclusion],[enables/disables inclusion of /ldcg],,[enable_auto_ldcg_inclusion="yes"])])

dnl======================================================================
dnl AC_LDAS_WITH_MOUNTPT
dnl======================================================================

AC_DEFUN([AC_LDAS_WITH_MOUNT_PT],
[AC_ARG_WITH(
	mountpt, 
	[  --with-mountpt          Specify frame builder directory],
	[ 
		MOUNT_PT="$mountpt";
	],[
		MOUNT_PT="/imports/frames";
	])
	AC_SUBST(MOUNT_PT)
])

dnl======================================================================
dnl AC_LDAS_WITH_CCMALLOC
dnl======================================================================

AC_DEFUN([AC_LDAS_WITH_CCMALLOC],
[AC_ARG_WITH(
	ccmalloc, 
	[  --with-ccmalloc         Link to ccmalloc when running tests],
	[ AC_CHECK_LIB( 
		ccmalloc,
		main,
		[ EXTRA_TEST_FLAGS='-lccmalloc -ldl' ],
		[ AC_MSG_WARN( ccmalloc not found )  ])
	])
	AC_SUBST(EXTRA_TEST_FLAGS)
])

dnl======================================================================
dnl AC_LDAS_WITH_EXTRA_CXXFLAGS
dnl======================================================================

AC_DEFUN([AC_LDAS_WITH_EXTRA_CXXFLAGS],
[AC_ARG_WITH(
	extra_cxxflags, 
	[  --with-extra-cxxflags   Additional C++ compiler flags],
	[ if test -n "${with_extra_cxxflags}"
	  then
	    AC_LDAS_VAR_APPEND(CXXFLAGS,${with_extra_cxxflags},LITERAL)
	  fi
	],)
])

dnl======================================================================
dnl AC_LDAS_WITH_EXTRA_LDFLAGS
dnl======================================================================

AC_DEFUN([AC_LDAS_WITH_EXTRA_LDFLAGS],
[AC_ARG_WITH(
	extra_ldflags, 
	[  --with-extra-ldflags    Additional linker flags],
	[ if test -n "${with_extra_ldflags}"
	  then
	    LDFLAGS="$LDFLAGS ${with_extra_ldflags}";
	  fi
	],)
])


dnl======================================================================
dnl AC_LDAS_WITH_LDFLAGS
dnl======================================================================

AC_DEFUN([AC_LDAS_WITH_LDFLAGS],
[AC_ARG_WITH(
	ldflags, 
	[  --with-ldflags          Replace linker flags],
	[ if test -n "${with_ldflags}"
	  then
	    LDFLAGS="${with_ldflags}";
	  fi
	],)
])

dnl======================================================================
dnl AC_LDAS_WITH_LDAS_TARGET
dnl======================================================================

AC_DEFUN([AC_LDAS_WITH_LDAS_TARGET],
[AC_ARG_WITH(
	ldas_target,
	[  --with-ldas-target=[production|development|system_test]
                          Create target specific LDAS resource file ],
	[ case "${with_ldas_target}" in
	  production) BASE_PORT="11000";;
	  system_test) BASE_PORT="13000";;
	  development|*) BASE_PORT="12000";;
	  esac
	],[BASE_PORT="12000"])
	AC_SUBST(BASE_PORT)
])

dnl======================================================================
dnl AC_LDAS_WITH_PROFLE
dnl======================================================================

AC_DEFUN([AC_LDAS_WITH_PROFILE],
[AC_ARG_WITH(
	profile, 
	[  --with-profile=[prof|gprof] 
                          Compile for profiling ],
	[ if test "${with_profile}" = "prof"
	  then
	    CXXFLAGS="$CXXFLAGS -p";
	  elif  test "${with_profile}" = "gprof"
	  then
	    CXXFLAGS="$CXXFLAGS -pg";
	  else
	    AC_MSG_ERROR( Supported profiling modes: prof gprof. );
	  fi
	],)
])


dnl======================================================================
dnl AC_LDAS_ARG_WITH_OPTIMIZATION
dnl======================================================================

AC_DEFUN([AC_LDAS_ARG_WITH_OPTIMIZATION],
[AC_ARG_WITH(
	optimization,
	[  --with-optimization     Compiler optimization (extreme|high|medium|low|none)],
	[ ],[with_optimization="medium"])
	case x$CC in
	x) ;; dnl ignore because it has not been set
	*)
	  case x${with_optimization} in
	  xnone) dnl -- none
	    ldas_prog_cc_optimization=$ldas_prog_cc_optimization_none
            ;;
	  xextreme) dnl -- extreme
            ldas_prog_cc_optimization=$ldas_prog_cc_optimization_extreme
	    ;;
	  xhigh) dnl -- high
	    ldas_prog_cc_optimization=$ldas_prog_cc_optimization_high
   	    ;;
          xlow) dnl -- low
            ldas_prog_cc_optimization=$ldas_prog_cc_optimization_low
            ;;
	  xdefault) dnl -- default
            ldas_prog_cc_optimization=$ldas_prog_cc_optimization_default
            ;;
          *) dnl -- all other cases
	    ldas_prog_cc_optimization=$ldas_prog_cc_optimization_medium
	    ;;
	  esac
	  ;;
	esac
	case x$CXX in
	x) ;; dnl ignore because it has not been set
	*)
	  case x${with_optimization} in
	  xnone) dnl -- none
	    ldas_prog_cxx_optimization=$ldas_prog_cxx_optimization_none
            ;;
	  xextreme) dnl -- extreme
            ldas_prog_cxx_optimization=$ldas_prog_cxx_optimization_extreme
	    ;;
	  xhigh) dnl -- high
	    ldas_prog_cxx_optimization=$ldas_prog_cxx_optimization_high
   	    ;;
          xlow) dnl -- low
            ldas_prog_cxx_optimization=$ldas_prog_cxx_optimization_low
            ;;
	  xdefault) dnl -- default
            ldas_prog_cxx_optimization=$ldas_prog_cxx_optimization_default
            ;;
          *) dnl -- all other cases
	    ldas_prog_cxx_optimization=$ldas_prog_cxx_optimization_medium
	    ;;
	  esac
	  ;;
	esac
])

dnl----------------------------------------------------------------------
dnl Check for ldas packages
dnl AC_LDAS_CHECK_PACKAGE(SourceDir,LaFileName,[InstallDir])
dnl----------------------------------------------------------------------

AC_DEFUN([LDAS_CHECK_PACKAGE],
[
  if test -d ${ldas_top}/$1
  then
    ac_ldas_check_package_inc='[$][(]ldas_top_builddir[)]/include [$][(]ldas_top_builddir[)]/$1';
    ac_ldas_check_package_lib='[$][(]ldas_top_builddir[)]/$1/$2';
  elif test -d ${prefix}/include/$3 && test -d ${prefix}/lib/$3
  then
    ac_ldas_check_package_inc=${prefix}/include;
    ac_ldas_check_package_lib=${prefix}/lib/$3/$2;
  else
    ac_ldas_check_package_inc="";
    ac_ldas_check_package_lib="";
  fi
  #----------------------------------------------------------------------
  # Check to see if the package is available
  #----------------------------------------------------------------------
  define([ldas_tmp_var],dnl
	 patsubst(translit(patsubst($2,[^lib],[]),[A-Z],[a-z]),[[.]],[_]))
  if test -n "${ac_ldas_check_package_inc}" \
	&& test -n "${ac_ldas_check_package_lib}"
  then
    AC_LDAS_PREFIX_INCLUDE_PATH(${ac_ldas_check_package_inc})
    ldas_tmp_var=${ac_ldas_check_package_lib}
    AC_SUBST(ldas_tmp_var)
  fi
  undefine([ldas_tmp_var])
])

AC_DEFUN([AC_LDAS_CHECK_PACKAGE_DBACCESS],
[ LDAS_CHECK_PACKAGE(lib/dbaccess/src,libdbaccess.la)
  AM_CONDITIONAL(HAVE_PACKAGE_DBACCESS, test x$dbaccess_la != x)
])

AC_DEFUN([AC_LDAS_CHECK_PACKAGE_FILTERS],
[ LDAS_CHECK_PACKAGE(lib/filters/src,libfilters.la)
  AM_CONDITIONAL(HAVE_PACKAGE_FILTERS, test x$filters_la != x)
])

AC_DEFUN([AC_LDAS_CHECK_PACKAGE_FRAMECPP],
[ LDAS_CHECK_PACKAGE(lib/framecpp/src,libframecpp.la)
  AM_CONDITIONAL(HAVE_PACKAGE_FRAMECPP, test x$framecpp_la != x)
])

AC_DEFUN([AC_LDAS_CHECK_PACKAGE_GENERAL],
[ LDAS_CHECK_PACKAGE(lib/general/src,libgeneral.la)
  AM_CONDITIONAL(HAVE_PACKAGE_GENERAL, test x$general_la != x)
])

AC_DEFUN([AC_LDAS_CHECK_PACKAGE_ILWD],
[ LDAS_CHECK_PACKAGE(lib/ilwd/src,libilwd.la)
  AM_CONDITIONAL(HAVE_PACKAGE_ILWD, test x$ilwd_la != x)
])

AC_DEFUN([AC_LDAS_CHECK_PACKAGE_ILWDFCS],
[ LDAS_CHECK_PACKAGE(lib/ilwdfcs/src,libilwdfcs.la)
  AM_CONDITIONAL(HAVE_PACKAGE_ILWDFCS, test x$ilwdfcs_la != x)
])

AC_DEFUN([AC_LDAS_CHECK_PACKAGE_MIME],
[ LDAS_CHECK_PACKAGE(lib/mime/src,libmime.la)
  AM_CONDITIONAL(HAVE_PACKAGE_MIME, test x$mime_la != x)
])

AC_DEFUN([AC_LDAS_CHECK_PACKAGE_GENERIC_API],
[ LDAS_CHECK_PACKAGE(api/genericAPI/so/src,libgenericAPI.la,genericAPI)
  AM_CONDITIONAL(HAVE_PACKAGE_GENERICAPI, test x$genericapi_la != x)
  LDAS_CHECK_PACKAGE(api/genericAPI/so/src,libApiCommon.la)
  AM_CONDITIONAL(HAVE_PACKAGE_API_COMMON, test x$apicommon_la != x)
  LDAS_CHECK_PACKAGE(api/genericAPI/so/src,libApiCommonNS.la)
  AM_CONDITIONAL(HAVE_PACKAGE_API_COMMON_NS, test x$apicommonns_la != x)
  LDAS_CHECK_PACKAGE(api/genericAPI/so/src,libStat.la)
  AM_CONDITIONAL(HAVE_PACKAGE_STAT, test x$Stat_la != x)
  LDAS_CHECK_PACKAGE(api/genericAPI/so/src,libtcltid.la)
  AM_CONDITIONAL(HAVE_PACKAGE_TCL_TID, test x$tcltid_la != x)
])

dnl----------------------------------------------------------------------
dnl Check to see if ospace is available -
dnl variable:
dnl	$1 - a value other than "no" causes aborting of the configuration
dnl		process. A value of "no" indicates that Object Space is
dnl		an optional library.
dnl----------------------------------------------------------------------

AC_DEFUN([AC_LDAS_CHECK_LIB_OSPACE],
[AC_CHECK_LIB(socket,connect)
AC_CHECK_LIB(rt,sched_yield)
AC_CHECK_LIB(pthread,pthread_join)
AC_ARG_WITH(
	ospace, 
	[  --with-ospace[=DIR]         ObjectSpace directory],
	[ case x"$with_ospace" in
	  "x"|xno) ;;
	  *)
	    case x$enable_64bit in
	    xyes)
	      if test -f "${with_ospace}/lib/64/libospace${STLDEBUG}.la"
	      then
	        ospace_la="${with_ospace}/lib/64/libospace${STLDEBUG}.la"
	      elif test -f "${with_ospace}/lib64/libospace${STLDEBUG}.la"
	      then
	        ospace_la="${with_ospace}/lib64/libospace${STLDEBUG}.la"
	      else
	        ospace_la="${with_ospace}/lib/libospace${STLDEBUG}.la"
	      fi
	      ;;
	    *)
	      ospace_la="${with_ospace}/lib/libospace${STLDEBUG}.la"
	      ;;
	    esac
	    AC_SUBST(ospace_la)
	    case x$enable_64bit in
	    xyes)
	      if test -f "${with_ospace}/include64/ospace/config2.h"
	      then
	        AC_LDAS_PREFIX_INCLUDE_PATH("${with_ospace}/include64")
	      else
	        AC_LDAS_PREFIX_INCLUDE_PATH("${with_ospace}/include")
	      fi
	      ;;
	    *)
	      AC_LDAS_PREFIX_INCLUDE_PATH("${with_ospace}/include")
	      ;;
	    esac
	    ldas_tmp_cpp_flags_ospace=$CPPFLAGS
	    ldas_tmp_cxx_flags_ospace=$CXXFLAGS
	    CPPFLAGS="$LDAS_DEFS $CPPFLAGS"
	    CXXFLAGS="$LDAS_DEFS $CXXFLAGS"
	    AC_CHECK_HEADERS( 
		[ ospace/config2.h ],
		[ AC_DEFINE( HAVE_LIBOSPACE,1,[Defined if Object Space should be used] ) EXTRA_LIBS="${EXTRA_LIBS} -lospace${STLDEBUG}" ],
	        [  if test -n "$1" && test "$1" = "no"
		   then
			AC_MSG_WARN(Could not find ObjectSpace headers.)
                   else
			AC_MSG_ERROR(Could not find ObjectSpace headers.)
		   fi
	        ])
	    CPPFLAGS="$CPPFLAGS ${ldas_tmp_cpp_flags_ospace}"
	    CXXFLAGS="$CXXFLAGS ${ldas_tmp_cxx_flags_ospace}"
	    ;;
	 esac
	])
	AC_SUBST(EXTRA_LIBS)
])

dnl======================================================================
dnl Misc ...
dnl======================================================================

AC_DEFUN([AC_LDAS_LINK_CONFIG],
[ if test ! -f ${ldas_top}/include/$1/"`basename $2`"
  then
    if test ! -d "${ldas_top}/include/$1"
    then
      mkdir -p "${ldas_top}/include/$1"
    fi
    dest=${ldas_top}/include/$1/"`basename $2`"
    if test -f ${dest} -o -d ${dest} -o -h ${dest}
    then
	rm -rf ${dest}
    fi
    ln -s "`pwd`/$2" ${dest}
  fi
  AC_LDAS_PREFIX_INCLUDE_PATH('[$][(]ldas_top_builddir[)]/include')
])

dnl----------------------------------------------------------------------
dnl :TODO: This macro doesn't work
dnl AM_LDAS_VARIABLE_SET(VARIABLE [, ACTION-IF-FOUND
dnl	 [, ACTION-IF-NOT-FOUND]])
dnl----------------------------------------------------------------------

AC_DEFUN([AM_LDAS_VARIABLE_SET],
[ if test -n [$]$1;
  then if [$2]; then "$2"; fi
  else if [$3]; then "$3"; fi])


AC_DEFUN([AC_LDAS_CHECK_LIB],
[ ac_save_LIBS="$LIBS"
  LIBS="$1  $LIBS"
  cat > conftest.$ac_ext <<EOF
#line 2304 "configure"
#include "confdefs.h"

int main() {
$2()
; return 0; }
EOF
  if { (eval echo configure:2311: \"$ac_link\") 1>&5; (eval $ac_link) 2>&5; } && test -s conftest${ac_exeext}; then
    rm -rf conftest*
    $3
  else
    echo "configure: failed program was:" >&5
    cat conftest.$ac_ext >&5
    rm -rf conftest*
    $4
  fi
  rm -f conftest*
])


dnl ---------------------------------------------------------------------
dnl CUS_PATH_ODBC_DIRECT
dnl ---------------------------------------------------------------------

AC_DEFUN([CUS_PATH_ODBC_DIRECT],
[test -z "$odbc_direct_test_library" && odbc_direct_test_library=iodbc
test -z "$odbc_direct_test_func" && odbc_direct_test_func=SQLAllocEnv
test -z "$odbc_test_include" && odbc_direct_test_include=isql.h
  for ac_dir in		\
    /ldcg/include \
    ; \
  do
    if test -r "$ac_dir/$odbc_direct_test_include"; then
      no_odbc= ac_odbc_includes=$ac_dir;
      break;
    fi
  done

# Check for the libraries.
# See if we find them without any special options.
# Don't add to $LIBS permanently.

ac_save_LIBS="$LIBS"
LIBS="-l$odbc_direct_test_library $LIBS"
# First see if replacing the include by lib works.
for ac_dir in `echo "$ac_odbc_includes" | sed s/include/lib/` \
  /ldcg/lib \
  ; \
do
  for ac_extension in a so sl; do
    if test -r $ac_dir/lib${odbc_direct_test_library}.$ac_extension; then
      no_odbc= ac_odbc_libraries=$ac_dir
      break 2
    fi
  done
done
LIBS=$ac_save_LIBS])

dnl ---------------------------------------------------------------------
dnl AC_LDAS_ODBC_FILES
dnl ---------------------------------------------------------------------

AC_DEFUN([AC_LDAS_ODBC_FILES],
[AC_REQUIRE_CPP()dnl

AC_ARG_WITH(odbc-includes, [  --with-odbc-includes=DIR
                          ODBC include files are in DIR])
if test -z "$with_odbc_includes"; then
  odbc_includes=NONE
else
  odbc_includes=$with_odbc_includes
fi
AC_ARG_WITH(odbc-libraries, [  --with-odbc-libraries=DIR
                          ODBC library files are in DIR])
if test -z "$with_odbc_libraries"; then
  odbc_libraries=NONE
else
  odbc_libraries=$with_odbc_libraries
fi
AC_ARG_WITH(odbc-dm, [  --with-odbc-dm=ODBC_DRIVER_MANAGER
                          Specify the flavor of the ODBC Driver Manager (DB2 or IODBC)])
if test -z "$with_odbc_dm"; then
  odbc_dm=IODBC
else
  odbc_dm=$with_odbc_dm
fi
case ${odbc_dm} in
   DB2)
    ;;
   IODBC)
    ;;
   *)
	AC_MSG_ERROR(Unsupported ODBC Driver Manager \`${odbc_dm}'; --with-odbc-dm option);
	;;
  esac
ODBC_DM=$odbc_dm
AC_SUBST(ODBC_DM)

AC_MSG_CHECKING(for ODBC)
AC_ARG_WITH(odbc, [  --with-odbc             enable ODBC tests])
if test "x$with_odbc" = xno; then
  no_odbc=yes
else
  if test "x$odbc_includes" != xNONE && test "x$odbc_libraries" != xNONE; then
    no_odbc=
  else
    AC_CACHE_VAL(ac_cv_path_odbc,
      [# One or both of these vars are not set, and ther is no cached value.
      no_odbc=yes
      CUS_PATH_ODBC_DIRECT

      if test "$no_odbc" = yes; then
        ac_cv_path_odbc="no_odbc=yes"
      else
        ac_cv_path_odbc="no_odbc= ac_odbc_includes=$ac_odbc_includes ac_odbc_libraries=$ac_odbc_libraries"
      fi
      ])dnl
  fi
  eval "$ac_cv_path_odbc"
fi

if test "$no_odbc" = yes; then
  AC_MSG_RESULT(no)
  if  test "$odbc_dm" = "IODBC" ; then
    AC_MSG_ERROR(ODBC files not found)
  fi
else
  test "x$odbc_includes" = xNONE && odbc_includes=$ac_odbc_includes
  test "x$odbc_libraries" = xNONE && odbc_libraries=$ac_odbc_libraries
  ac_cv_path_odbc="no_odbc= ac_odbc_includes=$odbc_includes ac_odbc_libraries=$odbc_libraries"
  AC_MSG_RESULT([libraries $odbc_libraries, headers $odbc_includes])
fi

if test -n "$odbc_includes"; then
  IODBC_CFLAGS="-DIODBC -I$odbc_includes"
fi
AC_SUBST(IODBC_CFLAGS)
if test -n "$odbc_libraries"; then
  IODBC_LDFLAGS="-Wl,${RPATH},$odbc_libraries -L$odbc_libraries"
  IODBC_LIBS="-liodbc"
fi
AC_SUBST(IODBC_LDFLAGS)
AC_SUBST(IODBC_LIBS)
])

dnl ---------------------------------------------------------------------
dnl CUS_PATH_DB2_DIRECT
dnl ---------------------------------------------------------------------

AC_DEFUN([CUS_PATH_DB2_DIRECT],
[
test -z "$db2_direct_test_func" && db2_direct_test_func=SQLAllocEnv
test -z "$db2_test_include" && db2_direct_test_include=sqlcli.h
inc_dir="include"
lib_dir="lib"
db2_root_dirs="\
    /ldcg \
    /ldcg/IBMdb2 \
    /opt/IBMdb2/V6.1 \
    /opt/IBMdb2/V5.0"

  for dir in $db2_root_dirs
  do
    if test x$enable_64bit = xyes \
      -a -r "$dir/${inc_dir}64/$db2_direct_test_include"
    then
      no_db2= ac_db2_includes=$dir/${inc_dir}64;
      break
    fi
    if test -r "$dir/$inc_dir/$db2_direct_test_include"
    then
      no_db2= ac_db2_includes=$dir/$inc_dir;
      break
    fi
  done

# Check for the libraries.
# See if we find them without any special options.
# Don't add to $LIBS permanently.

ac_save_LIBS="$LIBS"
# First see if replacing the include by lib works.
for dir in `echo "$ac_db2_includes" | sed s,/$inc_dir,,` $db2_root_dirs
do
  case ${dir} in
  */V5.0)
    db2_direct_test_library="odbc"
    ;;
  *)
    db2_direct_test_library="db2"
    ;;
  esac
  LIBS=${ac_save_LIBS}
  LIBS="-l$db2_direct_test_library $LIBS"
  for ac_extension in a so sl; do
    if test x$enable_64bit = xyes \
	-a -r $dir/${lib_dir}64/lib${db2_direct_test_library}.$ac_extension
    then
      no_db2= ac_db2_libraries=$dir/${lib_dir}64
      break 2
    fi
    if test -r $dir/$lib_dir/lib${db2_direct_test_library}.$ac_extension
    then
      no_db2= ac_db2_libraries=$dir/$lib_dir
      break 2
    fi
  done
done
LIBS=$ac_save_LIBS])

dnl ---------------------------------------------------------------------
dnl AC_LDAS_DB2_FILES
dnl ---------------------------------------------------------------------

AC_DEFUN([AC_LDAS_DB2_FILES],
[AC_REQUIRE_CPP()dnl

AC_ARG_WITH(db2-includes, [  --with-db2-includes=DIR DB2 include files are in DIR])
if test -z "$with_db2_includes"; then
  db2_includes=NONE
else
  db2_includes=$with_db2_includes
fi
AC_ARG_WITH(db2-libraries, [  --with-db2-libraries=DIR
                          DB2 library files are in DIR])
if test -z "$with_db2_libraries"; then
  db2_libraries=NONE
else
  db2_libraries=$with_db2_libraries
fi

AC_MSG_CHECKING(for DB2)
AC_ARG_WITH(db2, [  --with-db2              enable DB2 tests])
if test "x$with_db2" = xno; then
  no_db2=yes
else
  if test "x$db2_includes" != xNONE && test "x$db2_libraries" != xNONE; then
    no_db2=
  else
    AC_CACHE_VAL(ac_cv_path_db2,
      [# One or both of these vars are not set, and ther is no cached value.
      no_db2=yes
      CUS_PATH_DB2_DIRECT

      if test "$no_db2" = yes; then
        ac_cv_path_db2="no_db2=yes"
      else
        ac_cv_path_db2="no_db2= ac_db2_includes=$ac_db2_includes ac_db2_libraries=$ac_db2_libraries"
      fi
      ])dnl
  fi
  eval "$ac_cv_path_db2"
fi

if test "$no_db2" = yes; then
  AC_MSG_RESULT(no)
  if  test "$odbc_dm" = "DB2" ; then
    AC_MSG_ERROR(DB2 files not found)
  fi
else
  test "x$db2_includes" = xNONE && db2_includes=$ac_db2_includes
  test "x$db2_libraries" = xNONE && db2_libraries=$ac_db2_libraries
  ac_cv_path_db2="no_db2= ac_db2_includes=$db2_includes ac_db2_libraries=$db2_libraries"
  AC_MSG_RESULT([libraries $db2_libraries, headers $db2_includes])
fi

if test -n "$db2_includes"; then
  DB2_CFLAGS="-DDB2 -I$db2_includes"
fi
AC_SUBST(DB2_CFLAGS)
if test -n "$db2_libraries"
then
  DB2_LDFLAGS="-Wl,${RPATH},$db2_libraries -L$db2_libraries"
  case "${DB2_LDFLAGS}" in
  */V5.0/*)
    DB2_LIBS="-lodbc -lodbcinst"
    ;;
  *)
    DB2_LIBS="-ldb2"
    ;;
  esac
fi
if test "x$db2_libraries" = xNONE; then
      DB2_LIBS_DIR=
else
      DB2_LIBS_DIR=$db2_libraries
fi
AC_SUBST(DB2_LIBS_DIR)
AC_SUBST(DB2_LDFLAGS)
AC_SUBST(DB2_LIBS)
])


dnl ---------------------------------------------------------------------
dnl AC_LDAS_DATABASE_FILES
dnl ---------------------------------------------------------------------

AC_DEFUN([AC_LDAS_DATABASE_FILES],
[
  AC_ARG_ENABLE(metadata-api,[  --enable-metadata-api[=yes]
                          build the metadata API],
  [enable_metadata="no"], enable_metadata="yes")
  case "${enable_metadata}" in
    no)
	AC_SUBST(DB2_LIBS_DIR)
	AC_SUBST(DB2_LDFLAGS)
	AC_SUBST(DB2_LIBS)
	;;
    *)	AC_LDAS_ODBC_FILES
	AC_LDAS_DB2_FILES
	;;
  esac
])


dnl =====================================================================
dnl   General routine for searching out supported languages
dnl =====================================================================
AC_DEFUN([LDAS_PACKAGE_LANGUAGE],
[
  AC_ARG_WITH([$1],[  --with-$1=<dir>  Directory where $1 is installed],
    [
    ],
    [
      dnl ---------------------------------------------------------------
      dnl  Determine subdir base on file type
      dnl ---------------------------------------------------------------
      case $3 in
      *.h) subdir=include;;
      esac
      dnl ---------------------------------------------------------------
      dnl  Making an educated guess as to where Language is installed
      dnl ---------------------------------------------------------------
      for path in `echo $PATH | \
	  sed -e 's/s*bin:/'$subdir':/g' -e 's/:/ /g' -e 's/s*bin$/'$subdir'/'`
      do
	if test $2 $path/$3
	then
	  with_$1="$path"
	  break
        fi
      done
    ])
  AC_MSG_CHECKING(for $1 installation)
  case x${with_$1} in
  x|xno)
    AC_MSG_RESULT(no)
    with_$1="" # Normalize the result
    ;;
  *)
    if test $2 ${with_$1}/$3
    then
      AC_MSG_RESULT(${with_$1})
      AC_DEFINE(translit(HAVE_$1,[a-z],[A-Z]),dnl
	        [1],dnl
                [Defined if the scripting language $1 is supported])
    else
      AC_MSG_ERROR([no (${with_$1})])
    fi
    ;;
  esac
])
dnl =====================================================================
dnl  Look for TCL scripting language support
dnl =====================================================================
AC_DEFUN([LDAS_PACKAGE_TCL],
[
  AC_PATH_PROG( TCLSH, tclsh )
  AC_PATH_PROG( WISH, wish )
  AC_ARG_WITH([tcl-config],[  --with-tcl-config=<file>  Fully pathed location of the TCL configuration file],
    [],
    [
      dnl ---------------------------------------------------------------
      dnl  Making an educated guess as to where TCL is installed
      dnl ---------------------------------------------------------------
      for path in `echo $PATH | \
        sed -e 's/bin\([[^:]]*\):/lib\1 /g' -e 's/:/ /g' -e 's/bin\(.*\)$/lib\1/g'`
      do
        AC_MSG_WARN(looking at $path/tclConfig.sh)
        if test -f $path/tclConfig.sh
        then
          with_tcl_config=$path/tclConfig.sh
	  break
        fi
      done
    ])
  AM_CONDITIONAL(HAVE_TCL,test -f "${with_tcl_config}")
  AC_MSG_CHECKING(for tcl)
  if test -f "${with_tcl_config}"
  then
    AC_MSG_RESULT(yes)
    dnl -----------------------------------------------------------------
    dnl   Compute the installation directory of the libraries
    dnl -----------------------------------------------------------------
    tcl_lib_dir=`grep TCL_LIB_SPEC= ${with_tcl_config} | \
      sed -e 's/^.*-L\([^ ]*\).*$/\1/g'`
    TCL_DBGX==`grep TCL_DBGX= ${with_tcl_config} | \
      sed -e 's/^.*=\(.*\)$/\1/g'`
    TCLSTUB_LIB=`grep TCL_STUB_LIB_FILE= ${with_tcl_config} | \
      sed -e 's/^.*='\''\(.*\)'\''.*$/\1/g'`
    dnl -----------------------------------------------------------------
    dnl  Compute the version of TCL being used
    dnl -----------------------------------------------------------------
    AC_MSG_CHECKING(for tcl version)
    TCL_VER=`grep TCL_VERSION= ${with_tcl_config} | \
      sed -e 's/^.*='\''\(.*\)'\''$/\1/g'`
    AC_MSG_RESULT($TCL_VER)
    AC_SUBST(TCL_VER)
    dnl -----------------------------------------------------------------
    dnl  Define for building extensions
    dnl -----------------------------------------------------------------
    AC_DEFINE([HAVE_TCL],dnl
	      [1],dnl
              [Defined if the scripting language TCL is supported])
  else
    AC_MSG_RESULT(no)
  fi
])
dnl =====================================================================
dnl  Look for Python scripting language support
dnl =====================================================================
AC_DEFUN([LDAS_PACKAGE_PYTHON],
[
  AC_PATH_PROG( PYTHON, python )
  AC_PATH_PROG( PYTHON_CONFIG, python-config )
  if ! test -f "$PYTHON"
  then
    unset PYTHON
  fi
  case x${PYTHON} in
  x)
    unset PYTHON
    ;;
  *)
    dnl -----------------------------------------------------------------
    dnl  Figure out which version of python is installed
    dnl -----------------------------------------------------------------
    PYTHON_VERSION="`${PYTHON} -V 2>&1 | \
      sed 's,^Python \([[0-9]][[0-9]]*\(\.[[0-9]][[0-9]]*\)*\).*$,\1,gi' | \
      sed 's,^\([[0-9]][[0-9]]*\(\.[[0-9]][[0-9]]*\)\).*$,\1,gi'`"
    case x$PYTHON_CONFIG in
    x)
	dnl -------------------------------------------------------------
	dnl Get the information the hard way
	dnl -------------------------------------------------------------
        PYTHON_INCLUDES="-I`echo ${PYTHON} | \
	  sed 's,/python$,,g' | \
	  sed 's,/amd64,64,g' | \
	  sed 's,/64,64,g' | \
	  sed 's,/bin\([[^/]]*\),/include\1,g'`/python$PYTHON_VERSION";
	;;
    *)
	dnl -------------------------------------------------------------
	dnl Newer version of python have the python-config script
	dnl -------------------------------------------------------------
    	PYTHON_INCLUDES="`$PYTHON_CONFIG --includes`"
	;;
    esac
    AC_SUBST(PYTHON_VERSION)
    AC_SUBST(PYTHON_INCLUDES)
    AC_SUBST(PYTHON)
  esac
  AC_MSG_CHECKING(for python headers)
  AM_CONDITIONAL(HAVE_PYTHON,test -n "${PYTHON_INCLUDES}")
])
